from transformers import AutoModelForCausalLM, AutoTokenizer
from transformers.generation import GenerationConfig
from qwen_model.modeling_qwen import QWenModel, QWenLMHeadModel

def load_model(model_path: str, generation_config_path: str=None, tokenizer_path: str=None, device_map: str="auto"):
    if tokenizer_path is None:
        tokenizer_path = model_path

    if generation_config_path is None:
        generation_config_path = model_path

    model_cls = QWenLMHeadModel # if 'chat' in model_path.lower() else QWenModel

    tokenizer = AutoTokenizer.from_pretrained(tokenizer_path, trust_remote_code=True)
    tokenizer.pad_token = tokenizer.eos_token = '<|endoftext|>'

    generation_config = GenerationConfig.from_pretrained(generation_config_path, trust_remote_code=True)

    generation_config.max_new_tokens = 1024

    model = model_cls.from_pretrained(model_path, device_map=device_map, trust_remote_code=True, bf16=True)
    model.generation_config = generation_config
    model.eval()
    print(model.generation_config)
    print(tokenizer)
    return model, tokenizer